#!/usr/bin/env python

# This application is released under the GNU General Public License
# v3 (or, at your option, any later version). You can find the full
# text of the license under http://www.gnu.org/licenses/gpl.txt.
# By using, editing and/or distributing this software you agree to
# the terms and conditions of this license.
# Thank you for using free software!

#  Sunbird Screenlet (c) Jan Badenhorst 2008 <janhendrik.badenhorst@gmail.com>
#
# INFO:
# -
#
# TODO: Lots of clean up!
# -

import screenlets, math
from screenlets.options import StringOption, TimeOption, AccountOption, FontOption, ColorOption, IntOption, BoolOption, ListOption # They types of fields availabel in the options menu
import cairo, pango, gobject
import gtk
import urllib2
from urllib2 import urlopen
import urllib
import decimal
import re
import gconf
import time
import datetime
from sys import argv

class SunbirdScreenlet(screenlets.Screenlet):
	"""
	Sunbird displays personal weather station Info.
	It uses the "realtime" functionality as described for Cumulus PWS Software
	by Steve from Sandaysoft (www.sandaysoft.com) and implemented for Cumulus 1.8.3
	It also requres the user to use Cumulus's auto-upload functionality, where a 2nd
	set of data is posted. The format of this data is pipe-delimited ("|") text.
	A sample of this file is included in this package in the "Cumulus_Data_Upload" directory, and
	is called "sunbird.txt". This file should be placed in Cumulus's "web" directory, and Cumulus should
	be set to upload this periodically. This is in addition to the realtime.txt file that should
	also be up-loaded by Cumulus by checking the "Enable Realtime" check box.
	The remote location and file name for both of these files can be set via Sunbird's
	options menu.
	Thanks to Steve and the Sandaysoft crowd on great free software.

	In order to get the conditions of your location Sunbird scrapes Weather Underground
	using your stations longitude and lattitude to determine the weather conditions in your area.
	Thanks WU Team.

	To display the moon pahse, I used both the code and the assets from
	Marcus Furius's FuriusMoonScreenlet (marcus_furius@hotmail.com)\n visit http://www.marcus-furius.com
	Thanks Marcus.
	"""

	# Default meta-info for Screenlets
	__name__	= 'SunbirdScreenlet'
	__version__	= '0.85'
	__author__	= 'Jan Badenhorst'
	__desc__	= __doc__

	# System directory - used to find theme images
	srcdir = argv[0][:argv[0].find('SunbirdScreenlet.py')].strip()

	# Always-on panel size
	current_width = 280
	clear_space_top_y = 50
	clear_space_left_x = 50
	always_on_data_size_y = 60


	# Set initial panel sizes
	wind_panel_height = 170
	humidity_panel_height = 50
	barometer_panel_height = 50
	conditions_panel_height = 20
	temperature_panel_height = 50
	precipitation_panel_height = 50
	indoor_conditions_panel_height = 50
	precipitation_history_panel_height = 50
	initial_height = clear_space_top_y + always_on_data_size_y
	current_height = initial_height + conditions_panel_height + wind_panel_height + barometer_panel_height + humidity_panel_height + temperature_panel_height + precipitation_panel_height + indoor_conditions_panel_height + precipitation_history_panel_height

	# Set initial panel disply status
	show_wind_panel = False
	show_humidity_panel = False
	show_barometer_panel = False
	show_conditions_panel = False
	show_temperature_panel = False
	show_precipitation_panel = False
	show_indoor_conditions_panel = False
	show_precipitation_history_panel = False

	## URL Stuff
	url = 'http://www.faqawi.com/sunbird/realtime.txt'
	url_slow = 'http://www.faqawi.com/sunbird/sunbird.txt'

	# Internals
	ang = 0
	number = 0
	step_angle = 5
	target_angle = 0
	rotation_angle = 0
	back_image = "Unknown"
	conditions_image = "Unknown"

	ampr = 0
	maxw = 0
	maxwH = 0
	maxwM = 0
	wdir = "---"
	rain_today = 0
	feels_data = 0
	wind_chill = 0
	heat_index = 0
	name = "Sunbird"
	conditions = "---"
	barometer_rate = 0
	dew_temperature = 0
	max_temperature = 0
	indoor_humidity = 0
	yearly_rainfall = 0
	monthly_rainfall = 0
	lastupdate_date = ""
	lastupdate_time = ""
	relative_humidity = 0
	ten_min_high_gust = 0
	relative_humidity = 0
	current_rain_rate = 0
	current_barometer = 0
	indoor_temperature = 0
	current_temperature = 0
	minimum_temperature = 0
	time_max_temperature = 0
	feels_lable = "Feels Like"
	time_minimum_temperature = 0
	latest_wind_speed_reading = 0
	verbose_date_time = "Not Connected"

	# Units
	rain_unit = "mm"
	wind_unit = "km/h"
	pressure_unit = "mb"
	temperature_unit = u'\u00b0' + 'C'

	# Possable condition descriptors from Weatherunderground
	Conditions = {
	'':'Unknown',
	'A Few Clouds and Breezy':'Few_Clouds_Breezy',
	'A Few Clouds and Windy':'Few_Clouds_Breezy',
	'A Few Clouds with Haze':'Partly_Clear',
	'A Few Clouds':'Partly_Clear',

	'Blowing Dust':'Dust',
	'Blowing Sand':'Dust',
	'Blowing Snow':'Blowing_Snow',
	'Blowing Snow in Vicinity':'Blowing_Snow',

	'Chance of Flurries':'Flurries',
	'Chance of Rain':'Rain',
	'Chance of Sleet':'Sleet',
	'Chance of Snow':'Snow',
	'Chance of Thunderstorms':'Thunderstorms',

	'Clear':'Unknown',
	'Clear and Breezy':'Breezy',
	'Clear with Haze':'Haze',

	'Cloudy':'Cloudy',

	'Drizzle':'Drizzle',
	'Drizzle Fog':'Drizzle',
	'Drizzle Ice Pellets':'Rain_and_Sleet',
	'Drizzle Mist':'Drizzle',
	'Drizzle Snow':'Rain_and_Snow',

	'Dust':'Dust',
	'Dust Storm':'Dust','Heavy Dust Storm':'Dust',
	'Dust Storm in Vicinity':'Dust',
	'Dust Whirls':'Dust',
	'Dust Whirls in Vicinity':'Dust',

	'Fair':'Unknown',
	'Fair and Breezy':'Breezy',
	'Fair and Windy':'Windy',
	'Fair with Haze':'Haze',

	'Flurries':'Flurries',

	'Fog':'Fog',
	'Fog in Vicinity':'Fog',

	'Freezing Drizzle':'Freezing_Drizzle',
	'Freezing Drizzle Rain':'Freezing_Drizzle',
	'Freezing Drizzle Snow':'Rain_and_Snow',
	'Freezing Drizzle in Vicinity':'Freezing_Drizzle',

	'Freezing Fog':'Fog',
	'Freezing Fog in Vicinity':'Fog',

	'Freezing Rain':'Freezing_Rain',
	'Freezing Rain Rain':'Freezing_Rain',
	'Freezing Rain Snow':'Rain_and_Snow',
	'Freezing Rain in Vicinity':'Freezing_Rain',

	'Funnel Cloud':'Tornado',
	'Funnel Cloud in Vicinity':'Tornado',

	'Hail':'Hail',
	'Hail Showers':'Rain_and_Hail',

	'Haze':'Haze',

	'Heavy Blowing Snow':'Blowing_Snow',

	'Heavy Drizzle':'Drizzle',
	'Heavy Drizzle Fog':'Drizzle',
	'Heavy Drizzle Ice Pellets':'Rain_and_Sleet',
	'Heavy Drizzle Mist':'Drizzle',
	'Heavy Drizzle Snow':'Rain_and_Snow',

	'Heavy Freezing Drizzle':'Freezing_Rain',
	'Heavy Freezing Drizzle Rain':'Freezing_Drizzle',
	'Heavy Freezing Drizzle Snow':'Rain_and_Snow',

	'Heavy Freezing Fog':'Fog',

	'Heavy Freezing Rain':'Freezing_Rain',
	'Heavy Freezing Rain Rain':'Freezing_Rain',
	'Heavy Freezing Rain Snow':'Rain_and_Snow',

	'Heavy Ice Pellets':'Sleet',
	'Heavy Ice Pellets Drizzle':'Rain_and_Sleet',
	'Heavy Ice Pellets Rain':'Rain_and_Sleet',

	'Heavy Rain':'Rain',
	'Heavy Rain Fog':'Rain',
	'Heavy Rain Freezing Drizzle':'Freezing_Drizzle',
	'Heavy Rain Freezing Rain':'Freezing_Rain',
	'Heavy Rain Ice Pellets':'Rain_and_Sleet',
	'Heavy Rain Mist':'Rain',

	'Heavy Rain Showers':'Showers',
	'Heavy Rain Showers Fog':'Showers',
	'Heavy Rain Showers Mist':'Showers',

	'Heavy Rain Snow':'Rain_and_Snow',

	'Heavy Sand Storm':'Dust',

	'Heavy Showers Rain':'Showers',
	'Heavy Showers Rain Fog':'Showers',
	'Heavy Showers Rain Mist':'Showers',
	'Heavy Showers Snow':'Heavy_Snow',
	'Heavy Showers Snow Fog':'Heavy_Snow',
	'Heavy Showers Snow Mist':'Heavy_Snow',

	'Heavy Small Hail Pellets':'Hail',
	'Heavy Small Snow Pellets':'Hail',

	'Heavy Snow':'Heavy_Snow',
	'Heavy Snow Blowing Snow':'Blowing_Snow',
	'Heavy Snow Drizzle':'Rain_and_Snow',
	'Heavy Snow Fog':'Heavy_Snow',
	'Heavy Snow Freezing Drizzle':'Rain_and_Snow',
	'Heavy Snow Freezing Rain':'Rain_and_Snow',
	'Heavy Snow Grains':'Flurries',
	'Heavy Snow Low Drifting Snow':'Blowing_Snow',
	'Heavy Snow Mist':'Heavy_Snow',
	'Heavy Snow Rain':'Rain_and_Snow',
	'Heavy Snow Showers':'Heavy_Snow',
	'Heavy Snow Showers Fog':'Flurries',
	'Heavy Snow Showers Fog':'Heavy_Snow',
	'Heavy Snow Showers Fog/Mist':'Heavy_Snow',
	'Heavy Snow Showers Mist':'Heavy_Snow',

	'Heavy Thunderstorm Rain':'Heavy_Thunderstorm_Rain',
	'Heavy Thunderstorm Rain Fog':'Heavy_Thunderstorm_Rain',
	'Heavy Thunderstorm Rain Hail':'Heavy_Thunderstorm_Rain_Hail',
	'Heavy Thunderstorm Rain Hail Fog':'Heavy_Thunderstorm_Rain_Hail',
	'Heavy Thunderstorm Rain Hail Haze':'Heavy_Thunderstorm_Rain_Hail',
	'Heavy Thunderstorm Rain Hail Mist':'Heavy_Thunderstorm_Rain_Hail',
	'Heavy Thunderstorm Rain Haze':'Heavy_Thunderstorm_Rain',
	'Heavy Thunderstorm Rain Mist':'Heavy_Thunderstorm_Rain',
	'Heavy Thunderstorm Rain Small Hail Pellets':'Heavy_Thunderstorm_Rain_Hail',
	'Heavy Thunderstorm Rain Small Snow Pellets':'Heavy_Thunderstorm_Rain_Hail',

	'Heavy Thunderstorm Snow':'Thunderstorms_Snow',

	'Ice Crystals':'Sleet',

	'Ice Pellets':'Sleet',
	'Ice Pellets Drizzle':'Rain_and_Sleet',
	'Ice Pellets Rain':'Rain_and_Sleet',
	'Ice Pellets in Vicinity':'Sleet',

	'Light Drizzle':'Light_Drizzle',
	'Light Drizzle Fog':'Drizzle',
	'Light Drizzle Ice Pellets':'Rain_and_Sleet',
	'Light Drizzle Mist':'Drizzle',
	'Light Drizzle Snow':'Rain_and_Snow',

	'Light Freezing Drizzle':'Freezing_Drizzle',
	'Light Freezing Drizzle Rain':'Freezing_Drizzle',
	'Light Freezing Drizzle Snow':'Rain_and_Snow',
	'Light Freezing Fog':'Fog',
	'Light Freezing Rain':'Freezing_Drizzle',
	'Light Freezing Rain Rain':'Freezing_Rain',
	'Light Freezing Rain Snow':'Rain_and_Snow',

	'Light Ice Pellets':'Sleet',
	'Light Ice Pellets Drizzle':'Rain_and_Sleet',
	'Light Ice Pellets Rain':'Rain_and_Sleet',

	'Light Rain':'Light_Rain',
	'Light Rain Drizzle':'Showers',
	'Light Rain Drizzle Mist':'Showers',
	'Light Rain Fog':'Light_Rain',
	'Light Rain Freezing Drizzle':'Freezing_Drizzle',
	'Light Rain Freezing Rain':'Freezing_Rain',
	'Light Rain Ice Pellets':'Rain_and_Sleet',
	'Light Rain Mist':'Light_Rain',

	'Light Rain Showers':'Showers',
	'Light Rain Showers Fog':'Showers',
	'Light Rain Showers Mist':'Showers',

	'Light Rain Snow':'Rain_and_Snow',
	'Light Rain and Breezy':'Light_Rain_Breezy',

	'Light Showers Rain':'Showers',
	'Light Showers Rain Fog':'Showers',
	'Light Showers Rain Mist':'Showers',
	'Light Showers Rain Small Hail':'Freezing_Drizzle',

	'Light Showers Snow':'Snow_Showers',
	'Light Showers Snow Fog':'Scattered_Snow_Showers',
	'Light Showers Snow Mist':'Scattered_Snow_Showers',

	'Light Small Hail Pellets':'Hail',
	'Light Small Snow Pellets':'Hail',

	'Light Snow':'Snow',
	'Light Snow Blowing Snow':'Blowing_Snow',
	'Light Snow Drizzle':'Rain_and_Snow',
	'Light Snow Fog':'Blowing_Snow',
	'Light Snow Freezing Drizzle':'Rain_and_Snow',
	'Light Snow Freezing Rain':'Rain_and_Snow',
	'Light Snow Grains':'Flurries',
	'Light Snow Low Drifting Snow':'Blowing_Snow',
	'Light Snow Mist':'Snow',
	'Light Snow Rain':'Rain_and_Snow',
	'Light Snow Showers':'Snow_Showers',
	'Light Snow Showers Fog':'Flurries',
	'Light Snow Showers Mist':'Scattered_Snow_Showers',
	'Light Snow and Breezy':'Blowing_Snow',

	'Light Thunderstorm':'Thunderstorms',
	'Light Thunderstorm Rain':'Thunderstorms_Rain',
	'Light Thunderstorm Rain Fog':'Thunderstorms_Rain',
	'Light Thunderstorm Rain Hail':'Thunderstorms_Rain_Hail',
	'Light Thunderstorm Rain Hail Fog':'Thunderstorms_Rain_Hail',
	'Light Thunderstorm Rain Hail Haze':'Thunderstorms_Rain_Hail',
	'Light Thunderstorm Rain Hail Mist':'Thunderstorms_Rain_Hail',
	'Light Thunderstorm Rain Haze':'Thunderstorms_Rain',
	'Light Thunderstorm Rain Mist':'Thunderstorms_Rain',
	'Light Thunderstorm Rain Small Hail Pellets':'Thunderstorms_Rain_Hail',
	'Light Thunderstorm Rain Small Snow Pellets':'Thunderstorms_Rain_Hail',
	'Light Thunderstorm Snow':'Thunderstorms_Snow',

	'Low Drifting Dust':'Dust',
	'Low Drifting Sand':'Dust',
	'Low Drifting Snow':'Blowing_Snow',

	'Mist':'Fog',

	'Mostly Cloudy':'Mostly_Cloudy',
	'Mostly Cloudy and Breezy':'Moustly_Cloudy_Breezy',
	'Mostly Cloudy and Windy':'Partly_Cloudy_Breezy',
	'Mostly Cloudy with Haze':'Mostly_Cloudy',

	'Mostly Sunny':'Mostly_Clear',

	'N/A':'Unknown',
	'NULL':'Unknown',

	'Overcast':'Overcast',
	'Overcast and Breezy':'Overcast_Breezy',
	'Overcast and Windy':'Overcast_Breezy',
	'Overcast with Haze':'Overcast',

	'Partial Fog':'Fog',
	'Partial Fog in Vicinity':'Fog',

	'Partly Cloudy':'Partly_Cloudy',
	'Partly Cloudy and Breezy':'Partly_Cloudy_Breezy',
	'Partly Cloudy and Windy':'Moustly_Cloudy_Breezy',
	'Partly Cloudy with Haze':'Partly_Cloudy',

	'Partly Sunny':'Partly_Cloudy',

	'Patches of Fog':'Fog',
	'Patches of Fog in Vicinity':'Fog',

	'Rain':'Rain',
	'Rain Fog':'Rain',
	'Rain Freezing Drizzle':'Freezing_Drizzle',
	'Rain Freezing Rain':'Freezing_Rain',
	'Rain Ice Pellets':'Rain_and_Sleet',
	'Rain Mist':'Rain',

	'Rain Showers':'Showers',
	'Rain Showers Fog':'Showers',
	'Rain Showers Mist':'Showers',
	'Rain Showers in Vicinity':'Showers',
	'Rain Showers in Vicinity Fog':'Showers',
	'Rain Showers in Vicinity Mist':'Showers',

	'Rain Snow':'Rain_and_Snow',

	'Sand':'Dust',
	'Sand Storm':'Dust',
	'Sand Storm in Vicinity':'Dust',
	'Sand Whirls':'Dust',
	'Sand Whirls in Vicinity':'Dust',

	'Scattered Clouds':'Scattered_Clouds',
	'Scattered Thunderstorm':'Scattered_Thunderstorms',

	'Shallow Fog':'Fog',
	'Shallow Fog in Vicinity':'Fog',

	'Showers Hail':'Rain_and_Hail',
	'Showers Ice Pellets':'Rain_and_Sleet',
	'Showers Rain':'Showers',
	'Showers Rain Fog':'Showers',
	'Showers Rain Mist':'Showers',
	'Showers Rain Small Hail':'Showers_Rain_Small_Hail',
	'Showers Rain in Vicinity':'Showers',
	'Showers Rain in Vicinity Fog':'Showers',
	'Showers Rain in Vicinity Mist':'Showers',
	'Showers Snow':'Snow_Showers',
	'Showers Snow Fog':'Flurries',
	'Showers Snow Mist':'Scattered_Snow_Showers',
	'Showers in Vicinity':'Showers',
	'Showers in Vicinity Fog':'Showers',
	'Showers in Vicinity Haze':'Showers',
	'Showers in Vicinity Snow':'Rain_and_Snow',

	'Sleet':'Sleet',

	'Small Hail Pellets':'Hail',
	'Small Snow Pellets':'Hail',

	'Smoke':'Smoke',

	'Snow':'Snow',
	'Snow Blowing Snow':'Blowing_Snow',
	'Snow Drizzle':'Rain_and_Snow',
	'Snow Fog':'Snow',
	'Snow Freezing Drizzle':'Rain_and_Snow',
	'Snow Freezing Rain':'Rain_and_Snow',
	'Snow Grains':'Flurries',
	'Snow Low Drifting Snow':'Blowing_Snow',
	'Snow Mist':'Snow',
	'Snow Rain':'Rain_and_Snow',
	'Snow Showers':'Snow_Showers',
	'Snow Showers Fog':'Snow_Showers',
	'Snow Showers Mist':'Snow_Showers',
	'Snow Showers in Vicinity':'Rain_and_Snow',
	'Snow Showers in Vicinity Fog':'Rain_and_Snow',
	'Snow Showers in Vicinity Mist':'Rain_and_Snow',

	'Sunny':'Unknown',

	'Thunderstorm':'Thunderstorms',
	'Thunderstorm Fog':'Thunderstorms',
	'Thunderstorm Mist':'Thunderstorms',
	'Thunderstorm Hail':'Thunderstorms_Hail',
	'Thunderstorm Hail Fog':'Thunderstorms_Hail',
	'Thunderstorm Hail Mist':'Thunderstorms_Hail',
	'Thunderstorm Haze in Vicinity':'Scattered_Thunderstorms',
	'Thunderstorm Haze in Vicinity Hail':'Thunderstorms_Hail',
	'Thunderstorm Heavy Rain':'Thunderstorms_Heavy_Rain',
	'Thunderstorm Heavy Rain Fog':'Thunderstorms_Heavy_Rain',
	'Thunderstorm Heavy Rain Hail':'Heavy_Thunderstorm_Rain_Hail',
	'Thunderstorm Heavy Rain Hail Fog':'Heavy_Thunderstorm_Rain_Hail',
	'Thunderstorm Heavy Rain Hail Haze':'Heavy_Thunderstorm_Rain_Hail',
	'Thunderstorm Heavy Rain Hail Mist':'Heavy_Thunderstorm_Rain_Hail',
	'Thunderstorm Heavy Rain Haze':'Thunderstorms_Heavy_Rain',
	'Thunderstorm Heavy Rain Mist':'Thunderstorms_Heavy_Rain',
	'Thunderstorm Ice Pellets':'Rain_and_Sleet',
	'Thunderstorm Light Rain':'Thunderstorms_Rain',
	'Thunderstorm Light Rain Fog':'Thunderstorms_Rain',
	'Thunderstorm Light Rain Hail':'Thunderstorms_Rain_Hail',
	'Thunderstorm Light Rain Hail Fog':'Thunderstorms_Rain_Hail',
	'Thunderstorm Light Rain Hail Haze':'Thunderstorms_Rain_Hail',
	'Thunderstorm Light Rain Hail Mist':'Thunderstorms_Rain_Hail',
	'Thunderstorm Light Rain Haze':'Thunderstorms_Rain',
	'Thunderstorm Light Rain Mist':'Thunderstorms_Rain',
	'Thunderstorm Rain':'Thunderstorms_Rain',
	'Thunderstorm Rain Fog':'Thunderstorms_Rain',
	'Thunderstorm Rain Hail':'Thunderstorms_Rain_Hail',
	'Thunderstorm Rain Hail Fog':'Thunderstorms_Rain_Hail',
	'Thunderstorm Rain Hail Mist':'Thunderstorms_Rain_Hail',
	'Thunderstorm Rain Mist':'Thunderstorms_Rain',
	'Thunderstorm Rain Small Hail Pellets':'Thunderstorms_Rain_Hail',
	'Thunderstorm Rain Small Snow Pellets':'Thunderstorms_Rain_Hail',
	'Thunderstorm Showers':'Scattered_Thunderstorms',
	'Thunderstorm Showers in Vicinity':'Scattered_Thunderstorms',
	'Thunderstorm Showers in Vicinity Hail':'Thunderstorms_Rain_Hail',
	'Thunderstorm Small Hail Pellets':'Thunderstorms_Rain_Hail',
	'Thunderstorm Small Snow Pellets':'Thunderstorms_Rain_Hail',
	'Thunderstorm Snow':'Thunderstorms_Snow',
	'Thunderstorm in Vicinity':'Scattered_Thunderstorms',
	'Thunderstorm in Vicinity Fog':'Scattered_Thunderstorms',
	'Thunderstorm in Vicinity Hail':'Thunderstorms_Hail',
	'Thunderstorm in Vicinity Hail Fog':'Thunderstorms_Hail',
	'Thunderstorm in Vicinity Hail Haze':'Thunderstorms_Hail',
	'Thunderstorm in Vicinity Hail Mist':'Thunderstorms_Hail',
	'Thunderstorm in Vicinity Haze':'Scattered_Thunderstorms',
	'Thunderstorm in Vicinity Mist':'Scattered_Thunderstorms',

	'Tornado':'Tornado',

	'Unknown':'Unknown',

	'Water Spout':'Tornado',

	'Windy':'Windy',
	}

	# Options panel
	# Update interval
	# Make list over range with start index and increment
	update_choices = []
	update_choices.append (str(1) + " minute")
	update_choices.append (str(5) + " minutes")
	for x in range(15, 61, 15):
		update_choices.append (str(x) + " minutes")
	update_interval	= 200

	# Options panel
	# Initial Appearance
	# Values are editable through the UI
	text_colour_title = [0.87,0.87,0.87,1]
	text_colour_main_temperature = [0.87,0.87,0.87,1]
	text_colour_label = [1,0.815,0,1]
	text_colour_data = [0.87,0.87,0.87,1]
	text_colour_wind = [0.098,0.098,0.098,0.75]
	text_colour_wind_direction = [1,1,1,1]
	background_colour = [0.25,0.25,0.25,0.25]
	titlefont = "FreeSans Bold 20"
	titlesize = 20

	#---------------------------
	# The Options menu stuff
	#---------------------------
	def __init__ (self, **keyword_args):
		screenlets.Screenlet.__init__(self, width = self.current_width, height = self.current_height, **keyword_args)

		# Sunbird Data options
		self.add_options_group('Settings', 'Settings')
		self.add_option(StringOption('Settings', 'url', self.url,'Cumulus Realtime Data', 'This should point to the online realtime text file uploaded by Cumulus using the "Enable Realtime" functionality (usually called "realtime.txt")'),realtime=False)
		self.add_option(StringOption('Settings', 'url_slow', self.url_slow,'Cumulus Extra Data', 'This should point to the online text file that you set Cumulus to "Auto Update". It is usually called "sunbird.txt"'),realtime=False)
		self.add_option(StringOption('Settings', 'update_interval', self.update_interval,'Update Interval', 'Set in minutes', choices = self.update_choices),realtime=True)
		self.add_option(BoolOption('Settings','show_conditions_panel',self.show_conditions_panel, 'Show Conditions','Show conditions detail'),realtime=True)
		self.add_option(BoolOption('Settings','show_temperature_panel',self.show_temperature_panel, 'Show extra Temperature','Show extra temperature detail'),realtime=True)
		self.add_option(BoolOption('Settings','show_barometer_panel',self.show_barometer_panel, 'Show Barometer','Show barometer detail'),realtime=True)
		self.add_option(BoolOption('Settings','show_humidity_panel',self.show_humidity_panel, 'Show Humidity','Show humidity and dewpoint detail'),realtime=True)
		self.add_option(BoolOption('Settings','show_precipitation_panel',self.show_precipitation_panel, 'Show Presipitation','Show current precipitation data'),realtime=True)
		self.add_option(BoolOption('Settings','show_wind_panel',self.show_wind_panel, 'Show Wind','Show current wind conditions'),realtime=True)
		self.add_option(BoolOption('Settings','show_indoor_conditions_panel',self.show_indoor_conditions_panel, 'Show Indoor Conditions','Show indoor data'),realtime=True)
		self.add_option(BoolOption('Settings','show_precipitation_history_panel',self.show_precipitation_history_panel, 'Show Presipitation History','Rain this monty and year'),realtime=True)

		# Sunbird Appearance
		self.add_options_group('Appearance', 'Appearance')
		self.add_option(ColorOption('Appearance', 'text_colour_title', self.text_colour_title, 'Title Text colour', 'Colour of the Title text'),realtime=True)
		self.add_option(ColorOption('Appearance', 'text_colour_main_temperature', self.text_colour_main_temperature, 'Main Temperature colour', 'Colour of the always-on temperature text'),realtime=True)
		self.add_option(ColorOption('Appearance', 'text_colour_label', self.text_colour_label, 'Data Label colour', 'Colour of data labels'),realtime=True)
		self.add_option(ColorOption('Appearance', 'text_colour_data', self.text_colour_data, 'Data Text colour', 'Color of the data fields'),realtime=True)
		self.add_option(ColorOption('Appearance', 'text_colour_wind', self.text_colour_wind, 'Wind Speed Text colour', 'Color of the wind speed fields'),realtime=True)
		self.add_option(ColorOption('Appearance', 'text_colour_wind_direction', self.text_colour_wind_direction, 'Wind Direction Text colour', 'Color of the wind direction fields'),realtime=True)
		self.add_option(ColorOption('Appearance', 'background_colour', self.background_colour, 'Background Colour', 'Color of background'),realtime=True)

		self.add_option(FontOption('Appearance', 'titlefont',
			self.titlefont, 'Title Font',
			'The font of the text (when no Markup is used) ...'))

		# set theme
		self.theme_name = "DefaultTheme"
		self.theme_dir = self.theme_name

		# Add a 1 second (1000) TIMER
		self.timer = gobject.timeout_add( 1000, self.update)

	#---------------------------
	# When things start - do this
	#---------------------------
	def on_init (self):
		print "Sunbird has been initialized."
		self.add_menuitem("update", "Update now...")
		self.add_default_menuitems()
		self.update()

	#---------------------------
	# When a menu item is selected - do this
	#---------------------------
	def on_menuitem_select (self, id):
		"""
		Called when a menuitem is selected.
		"""
		if id == "update":
			self.set_to_zero("Updating", "Looking for data")
			self.redraw_canvas()
			self.number = -1
			self.update()
		pass

	#---------------------------
	# React to changes in the atribute tabs
	#---------------------------
	def on_after_set_atribute(self,name, value):
		"""
		Called after setting screenlet atributes
		"""
		pass

	#---------------------------
	# Re-loaded theme and fonts
	#---------------------------
	def on_load_theme (self):
		"""Called when the theme is reloaded (after loading, before redraw)."""
		self.set_font()

	#---------------------------
	# Interface Stuff
	#---------------------------
	def on_mouse_down (self, event):
		"""
		Called when a buttonpress-event occured in Screenlet's window.
		Returning True causes the event to be not further propagated.
		"""
		if event.button != 1 or event.type != gtk.gdk.BUTTON_PRESS:
			return False

	#---------------------------
	# Manage Update Cycle
	#---------------------------
	def update (self):
		"""
		This is called once a second from the timer object set in __init__
		Increments the variable self.number when called.
		If this value reaches the value set by user in self.update_interval
		the timer rolls over to 0
		At self.number == 0 we re-read the data from the web pages
		Once the data is read, the canvas is redrawn (with animation)
		Animation timer (10ms) is also controlled from here
		"""
		if self.number == 0:
			self.number = self.number+1
			self.check_url() # This gets and sets all the data variables
			gobject.timeout_add( 10, self.animate ) # This redraws the canvas
		elif self.number < self.to_seconds(self.update_interval):
			self.number = self.number+1
		else:
			self.number = 0
		return True

	#---------------------------
	# Set everything to zero
	#---------------------------
	def set_to_zero (self, message1,message2 ):
		self.name = message1
		self.verbose_date_time = message2
		self.current_temperature = 0
		self.relative_humidity = 0
		self.max_temperature = 0
		self.time_max_temperature = 0
		self.minimum_temperature = 0
		self.time_minimum_temperature = 0
		self.current_barometer = 0
		self.barometer_rate = 0
		self.dew_temperature = 0
		self.target_angle = 0
		self.wdir = "---"
		self.latest_wind_speed_reading = 0
		self.ten_min_high_gust = 0
		self.rain_today = 0
		self.current_rain_rate = 0
		self.monthly_rainfall = 0
		self.yearly_rainfall = 0
		self.feels_data = 0
		self.feels_lable = "Feels Like"
		self.wind_chill = 0
		self.ampr = 0
		self.heat_index = 0
		self.maxw = 0
		self.maxwH = 0
		self.maxwM = 0
		self.indoor_humidity = 0
		self.indoor_temperature = 0

	#---------------------------
	# Go online and get data
	#---------------------------
	def check_url (self):
		"""
		This routine opens 3 URLs for reading
		1) The realtime data URL reads the realtime.txt file
		   as posted by Cumulus realtime functionality.
		2) The semi-static data is a text file that the user should
		   place in Cumulus's "web" folder. The user should set Cumulus
		   to auto-upload this file periodically. We need this because
		   this presents info that is not (currently) presented in
		   Cumulu's realtime.txt file
		3) The conditions (or observations) are read from Weatherunderground
		   and uses the stations longitude and lattitude to establish the closest
		   observation to the station
		"""
		try:
			# 1) Realtime Data
			sburl = urllib2.Request(self.url)
			sb = urllib2.urlopen(sburl).read()
			self.read_realtime_data(sb)

			# 2) Semi-static Data
			slow_file = urllib2.Request(self.url_slow)
			slow_data = urllib2.urlopen(slow_file).read()
			self.read_semistatic_data(slow_data)

			# 3) Conditions Page - Weather Underground Lite
			req_cond = urllib2.Request(self.url_wund)
			res_cond = urllib2.urlopen(req_cond).read()
			self.read_conditions_page(res_cond)
		except:
			pass

	#---------------------------
	# Wind direction Animation
	#---------------------------
	def animate (self):
		"""
		This is where we redraw the canvas.
		Animates the wind direction indicator.
		Rotates in CW or ACW according to shortest
		path to target position
		"""
		if self.ang < self.target_angle:
			gap = self.target_angle - self.ang
			if gap > self.step_angle:
				self.ang = self.ang + (self.step_angle)
				self.redraw_canvas()
				return True
			else:
				self.ang = self.target_angle
				self.redraw_canvas()
				return False
		elif self.ang > self.target_angle:
			gap = self.ang - self.target_angle
			if gap > self.step_angle:
				self.ang = self.ang - (self.step_angle)
				self.redraw_canvas()
				return True
			else:
				self.ang = self.target_angle
				self.redraw_canvas()
				return False
		else:
			self.redraw_canvas()
			return False

	#---------------------------
	# Draw the screenlet
	#---------------------------
	def on_draw (self, ctx):
		"""
		Renders the widget
		"""
		# Set the various fonts and sizes
		text_font_normal = "FreeSans"
		text_font_bold	= "FreeSans Bold"
		fontsize = 10

		fs1_title = fontsize
		fs1_data = fontsize +10
		fs1_units = fontsize

		fs2_title = fontsize -3
		fs2_data = fontsize + 5
		fs2_units = fontsize - 3

		# if theme is loaded
		if self.theme:
			# set scale rel. to scale-attribute
			ctx.scale(self.scale, self.scale)

			# Set the background size
			self.current_height = self.initial_height
			if self.show_conditions_panel == True:
				self.current_height = self.current_height + self.conditions_panel_height
			if self.show_temperature_panel == True:
				self.current_height = self.current_height + self.temperature_panel_height
			if self.show_barometer_panel == True:
				self.current_height = self.current_height + self.barometer_panel_height
			if self.show_humidity_panel == True:
				self.current_height = self.current_height + self.humidity_panel_height
			if self.show_wind_panel == True:
				self.current_height = self.current_height + self.wind_panel_height
			if self.show_precipitation_panel == True:
				self.current_height = self.current_height + self.precipitation_panel_height
			if self.show_indoor_conditions_panel == True:
				self.current_height = self.current_height + self.indoor_conditions_panel_height
			if self.show_precipitation_history_panel == True:
				self.current_height = self.current_height + self.precipitation_history_panel_height

			# save the (0,0) screenlet origen context (ORIG)
			ctx.save()
			# Render Station Name and Timestamp
			self.draw_text(ctx, (self.width), 10, self.name, self.titlefont, self.titlesize , self.text_colour_title, 1)
			self.draw_text(ctx, (self.width), 35, self.verbose_date_time, text_font_normal, fontsize - 2, self.text_colour_title, 1)

			#-------------
			# move context from ORIG to top,left of BG (TLBG)
			#-------------
			ctx.translate (self.clear_space_left_x, self.clear_space_top_y)
			# save the TLBG context
			ctx.save()
			l_margin = self.clear_space_left_x
			r_margin = self.width - l_margin
			center_line = (self.width - self.clear_space_left_x)/2

			## Render the Background
			ctx.set_source_rgba(self.background_colour[0],self.background_colour[1],self.background_colour[2],self.background_colour[3],)
			self.draw_round_rect(ctx,0, 0, r_margin, self.current_height-self.clear_space_top_y, (20,10,10,10), True)

			# Render Main Temp Display
			ctx.save()
			ctx.translate(r_margin - 18 , 8)
			self.draw_text(ctx, 0, 0, "Temperature", text_font_normal, fs2_title, self.text_colour_label, 1)
			self.draw_text(ctx, 0, 13, self.temperature_unit, text_font_normal, fs1_units , self.text_colour_main_temperature, 1)
			self.draw_text(ctx, 0 - self.tsw, 13, str(self.current_temperature ), text_font_bold, fs1_data+3, self.text_colour_main_temperature, 1)
			ctx.restore()

			#-------------
			# Move context from TLBG to Top Left Data Fields (TLDF)
			#-------------
			ctx.translate(0,self.always_on_data_size_y)
			# save the TLDF context
			ctx.save()

			#-------------
			# Render Conditions
			#-------------
			if self.show_conditions_panel == True:
				self.theme['div.svg'].render_cairo(ctx)
				self.tsw = 0

				# image location
				ctx.save()

				ctx.translate(15, 0)
				self.draw_text(ctx, 0, 5, "Conditions: ", text_font_normal, fs2_title, self.text_colour_label, 0)
				self.draw_text(ctx, 50, 5, self.conditions, text_font_bold, fontsize-3, self.text_colour_data, 0)
				ctx.restore()

				# move context to bottom left of this data field
				ctx.translate(0, self.conditions_panel_height)


			#-------------
			# Render Temperature
			#-------------
			if self.show_temperature_panel == True:
				self.theme['div.svg'].render_cairo(ctx)
				self.tsw = 0

				# image location
				ctx.save()
				ctx.translate (center_line-20,8)

				self.theme['thermo.svg'].render_cairo(ctx)
				ctx.save()
				ctx.translate(40, 0)
				self.draw_text(ctx, 0, 0, self.feels_lable, text_font_normal, fs2_title, self.text_colour_label, 0)
				self.draw_text(ctx, 0, 13, str(self.feels_data), text_font_bold, fs2_data, self.text_colour_data, 0)
				self.draw_text(ctx, 0+ self.tsw, 13, self.temperature_unit, text_font_normal, fs2_units , self.text_colour_data, 0)
				ctx.restore()
				ctx.translate(0, 7)
				self.tsw = 0
				self.draw_text(ctx, 0, 0, " " + str(self.time_max_temperature), text_font_normal, fontsize-2, self.text_colour_data, 1)
				last = self.tsw
				self.draw_text(ctx, 0 - self.tsw, 0, self.temperature_unit, text_font_normal, fontsize-5, self.text_colour_data, 1)
				self.draw_text(ctx, 0 - last - self.tsw, 0, str(self.max_temperature), text_font_bold, fontsize-2, self.text_colour_data, 1)
				self.tsw = 0
				self.draw_text(ctx, 0, 13, " " + str(self.time_minimum_temperature), text_font_normal, fontsize-2, self.text_colour_data, 1)
				last = self.tsw
				self.draw_text(ctx, 0-self.tsw, 13, self.temperature_unit, text_font_normal, fontsize-5, self.text_colour_data, 1)
				self.draw_text(ctx, 0- last - self.tsw, 13, str(self.minimum_temperature), text_font_bold, fontsize-2, self.text_colour_data, 1)
				self.tsw = 0
				ctx.restore()
				# move context to bottom left of this data field
				ctx.translate(0, self.temperature_panel_height)

			#-------------
			# Render Barometer
			#-------------
			if self.show_barometer_panel == True:
				self.theme['div.svg'].render_cairo(ctx)
				self.tsw = 0

				# image location
				ctx.save()
				ctx.translate (center_line-20,8)

				if float(self.barometer_rate) < -0.05:
					self.theme['barom_neg.svg'].render_cairo(ctx)
				elif float(self.barometer_rate) > 0.05:
					self.theme['barom_pos.svg'].render_cairo(ctx)
				else:
					self.theme['barom.svg'].render_cairo(ctx)
				ctx.save()
				ctx.translate(40, 0)
				self.draw_text(ctx, 0, 0, "Barometer", text_font_normal, fs2_title, self.text_colour_label, 0)
				self.draw_text(ctx, 0, 13, str(self.current_barometer), text_font_bold, fs2_data, self.text_colour_data, 0)
				self.draw_text(ctx, 0 + self.tsw, 13, self.pressure_unit, text_font_normal, fs2_units, self.text_colour_data, 0)
				ctx.restore()
				self.tsw = 0
				self.draw_text(ctx, 0, 0, "Trend", text_font_normal, fs2_title, self.text_colour_label, 1)
				self.draw_text(ctx, 0 , 13, self.pressure_unit + "/h", text_font_normal, fs2_units, self.text_colour_data, 1)
				self.draw_text(ctx, 0- self.tsw, 13, str(self.barometer_rate), text_font_bold, fs2_data, self.text_colour_data, 1)
				ctx.restore()

				# move context to bottom left of this data field
				ctx.translate(0, self.barometer_panel_height)

			#-------------
			# Render Wind Direction
			#-------------
			if self.show_wind_panel == True:
				self.theme['div.svg'].render_cairo(ctx)

				# image location
				ctx.save()
				ctx.translate ( center_line, self.wind_panel_height/2)

				# RWD - The compass face
				ctx.save()
				img_size = 130/2
				ctx.translate(0-img_size,0-img_size)
				self.theme['compass.svg'].render_cairo(ctx)
				ctx.restore()

				# RWD - The wind speed
				ctx.save()
				ctx.translate(0,-26)
				self.tsw = 0
				self.draw_text(ctx, 0, 0, str(self.latest_wind_speed_reading),text_font_bold, fontsize+10, self.text_colour_wind, 2)
				self.draw_text(ctx, 0, 25, self.wind_unit ,text_font_normal, fontsize-3, self.text_colour_wind, 2)
				self.draw_text(ctx, 0, 35, "Gust ",text_font_normal, fontsize-3, self.text_colour_wind, 1)
				self.draw_text(ctx, 0, 35, self.ten_min_high_gust,text_font_bold, fontsize-2, self.text_colour_wind, 0)
				ctx.restore()

				# RWD - The wind direction arrow shadow
				ctx.save()
				ctx.translate(3,3)
				r = 170/2
				angle = math.pi*2*(self.ang)/360
				x = r*math.cos(-angle)
				y = r*math.sin(-angle)
				ctx.translate(-y-x,-x+y)
				ctx.rotate (angle)
				self.theme['wind_aguille-shadow.svg'].render_cairo(ctx)
				ctx.restore()

				# RWD - The wind direction arrow
				ctx.save()
				r = 170/2
				angle = math.pi*2*(self.ang)/360
				x = r*math.cos(-angle)
				y = r*math.sin(-angle)
				ctx.translate(-y-x,-x+y)
				ctx.rotate (angle)
				self.theme['wind_aguille.svg'].render_cairo(ctx)
				ctx.restore()

				# RWD - The wind direction Acronym
				# self.text_colour_wind_direction
				ctx.save()
				r = 135/2
				y = r*math.cos(-angle)
				x = r*math.sin(-angle)
				ctx.translate(-x,-y)
				self.draw_text(ctx, 0, (fontsize-3)/-2, str(self.ang),text_font_bold, fontsize-1, self.text_colour_wind_direction, 2)
				ctx.restore()

				# RWD - The wind direction Highlights
				# self.text_colour_wind_direction
				ctx.save()
				r = 135/2
				y = r*math.cos(-angle)
				x = r*math.sin(-angle)
				ctx.translate(-x-15,-y-15)
				self.theme['wind_aguille-highlights.svg'].render_cairo(ctx)
				ctx.restore()

				ctx.restore()
				# move context to bottom left of this data field
				ctx.translate(0, self.wind_panel_height)

			#-------------
			# Render Humidity
			#-------------
			if self.show_humidity_panel == True:
				self.theme['div.svg'].render_cairo(ctx)

				# image location
				ctx.save()
				ctx.translate (center_line-20, 8)

				self.tsw = 0
				self.theme['dew.svg'].render_cairo(ctx)
				ctx.save()
				ctx.translate(40, 0)
				self.draw_text(ctx, 0, 0, "Dew Point", text_font_normal, fs2_title, self.text_colour_label, 0)
				self.draw_text(ctx, 0, 13, str(self.dew_temperature), text_font_bold, fs2_data, self.text_colour_data, 0)
				self.draw_text(ctx, 0 + self.tsw, 13, self.temperature_unit, text_font_normal, fs2_units, self.text_colour_data, 0)
				ctx.restore()
				self.draw_text(ctx, 0, 0, "Humidity", text_font_normal, fs2_title, self.text_colour_label, 1)
				self.draw_text(ctx, 0 , 13, "%", text_font_normal, fs2_units, self.text_colour_data, 1)
				self.draw_text(ctx, 0- self.tsw, 13, str(self.relative_humidity), text_font_bold, fs2_data, self.text_colour_data, 1)
				ctx.restore()

				# move context to bottom left of this data field
				ctx.translate(0, self.humidity_panel_height)

			#-------------
			# Render Precipitation
			#-------------
			if self.show_precipitation_panel == True:
				self.theme['div.svg'].render_cairo(ctx)

				# image location
				ctx.save()
				ctx.translate (center_line-20, 8)

				self.tsw = 0
				self.theme['presip.svg'].render_cairo(ctx)
				ctx.save()
				ctx.translate(40, 0)
				self.draw_text(ctx, 0, 0, "Rain Today", text_font_normal, fs2_title, self.text_colour_label, 0)
				#self.draw_text(ctx, 0, 13, str(round(float(self.rain_today))), text_font_bold, fs2_data, self.text_colour_data, 0)
				self.draw_text(ctx, 0, 13, str(self.rain_today), text_font_bold, fs2_data, self.text_colour_data, 0)
				self.draw_text(ctx, 0 + self.tsw, 13, self.rain_unit, text_font_normal, fs2_units, self.text_colour_data, 0)
				ctx.restore()
				self.draw_text(ctx, 0, 0, "Rate Rain", text_font_normal, fs2_title, self.text_colour_label, 1)
				self.draw_text(ctx, 0 , 13, self.rain_unit + "/h", text_font_normal, fs2_units, self.text_colour_data, 1)
				#self.draw_text(ctx, 0- self.tsw, 13, str(round(float(self.current_rain_rate))), text_font_bold, fs2_data, self.text_colour_data, 1)
				self.draw_text(ctx, 0- self.tsw, 13, str(self.current_rain_rate), text_font_bold, fs2_data, self.text_colour_data, 1)
				ctx.restore()

				# move context to bottom left of this data field
				ctx.translate(0, self.precipitation_panel_height)

			#-------------
			# Render Indoor
			#-------------
			if self.show_indoor_conditions_panel == True:
				self.theme['div.svg'].render_cairo(ctx)

				# image location
				ctx.save()
				ctx.translate (center_line-20, 8)

				self.tsw = 0
				self.theme['indoor.svg'].render_cairo(ctx)
				ctx.save()
				ctx.translate(40, 0)
				self.draw_text(ctx, 0, 0, "Indoor Temperature", text_font_normal, fs2_title, self.text_colour_label, 0)
				self.draw_text(ctx, 0, 13, str(self.indoor_temperature), text_font_bold, fs2_data, self.text_colour_data, 0)
				self.draw_text(ctx, 0 + self.tsw, 13, self.temperature_unit, text_font_normal, fs2_units, self.text_colour_data, 0)
				ctx.restore()
				self.draw_text(ctx, 0, 0, "Indoor Humidity", text_font_normal, fs2_title, self.text_colour_label, 1)
				self.draw_text(ctx, 0 , 13, "%", text_font_normal, fs2_units, self.text_colour_data, 1)
				self.draw_text(ctx, 0- self.tsw, 13, str(self.indoor_humidity), text_font_bold, fs2_data, self.text_colour_data, 1)
				ctx.restore()

				# move context to bottom left of this data field
				ctx.translate(0, self.indoor_conditions_panel_height)

			#-------------
			# Render Precipitation History
			#-------------
			if self.show_precipitation_history_panel == True:
				self.theme['div.svg'].render_cairo(ctx)

				# image location
				ctx.save()
				ctx.translate (center_line-20, 8)

				self.tsw = 0
				self.theme['history.svg'].render_cairo(ctx)
				ctx.save()
				ctx.translate(40, 0)
				self.draw_text(ctx, 0, 0, "Rain this Year", text_font_normal, fs2_title, self.text_colour_label, 0)
				self.draw_text(ctx, 0, 13, str(self.yearly_rainfall), text_font_bold, fs2_data, self.text_colour_data, 0)
				self.draw_text(ctx, 0 + self.tsw, 13, self.rain_unit, text_font_normal, fs2_units, self.text_colour_data, 0)
				ctx.restore()
				self.draw_text(ctx, 0, 0, "Rain this Month", text_font_normal, fs2_title, self.text_colour_label, 1)
				self.draw_text(ctx, 0 , 13, self.rain_unit, text_font_normal, fs2_units, self.text_colour_data, 1)
				self.draw_text(ctx, 0- self.tsw, 13, str(self.monthly_rainfall), text_font_bold, fs2_data, self.text_colour_data, 1)
				ctx.restore()

				# move context to bottom left of this data field
				ctx.translate(0, self.precipitation_history_panel_height)

			#-------------
			# Restore Context TLDF
			#-------------
			ctx.restore()

			#-------------
			# Restore Context TLBG
			#-------------
			ctx.restore()

			#-------------
			# Restore Context ORIG
			#-------------
			ctx.restore()

			#-------------
			# Render Main Image
			#-------------
			ctx.save()
			ctx.translate(15, 15)
			try:
				src = cairo.ImageSurface.create_from_png(self.back_image)
				self.src_height =  src.get_height()
				self.src_width = src.get_width()
				ctx.set_source_surface(src, 0, 0)
				ctx.paint()

			except:
				pass
			ctx.restore()

			#-------------
			# Render Conditions
			#-------------
			ctx.save()
			#ctx.translate(-25, -30)
			ctx.translate(0, 0)
			try:
				src = cairo.ImageSurface.create_from_png(self.conditions_image)
				src_height =  src.get_height()
				src_width = src.get_width()
				ctx.set_source_surface(src, 0, 0)
				ctx.paint()
			except:
				pass
			ctx.restore()

	#---------------------------
	# GET THE WHOLE THING GOING
	#---------------------------
	def on_draw_shape (self, ctx):
		self.on_draw(ctx)

	#---------------------------
	# Read the realtime data
	#---------------------------
	def read_realtime_data (self, line):
		"""
		Reads the realtime.txt file posted by Cumulus
		Splits the data fields on a space
		This may be dangerous as the realtime.txt format is changeable
		"""
		dataposition = line.split()

		# Populate variables
		self.lastupdate_date = dataposition[0] #16:43:25
		self.lastupdate_time = dataposition[1] #16:43:25
		self.now = datetime.datetime(*time.strptime(dataposition[0] + dataposition[1], "%d/%m/%y%H:%M:%S")[0:6])
		self.verbose_date_time = self.now.strftime("%H:%M on %a %d %b %y")
		self.current_temperature = dataposition[2]
		self.relative_humidity = dataposition[3]
		self.dew_temperature = dataposition[4]
		self.wind_speed_average = dataposition[5]
		self.latest_wind_speed_reading = dataposition[6]
		self.target_angle = int(dataposition[7])
		self.current_rain_rate = dataposition[8]
		self.rain_today = dataposition[9]
		self.current_barometer = dataposition[10]
		self.wind_direction = dataposition[11]
		self.wind_speed_beaufort = dataposition[12]
		self.wind_unit = dataposition[13]
		self.temperature_unit = u'\u00b0' + dataposition[14]
		self.pressure_unit = dataposition[15]
		self.rain_unit = dataposition[16]
		self.wind_run_today = dataposition[17]
		self.barometer_rate = dataposition[18]
		self.monthly_rainfall = dataposition[19]
		self.yearly_rainfall = dataposition[20]
		self.yesterday_rainfall = dataposition[21]
		self.indoor_temperature = dataposition[22]
		self.indoor_humidity = dataposition[23]
		self.wind_chill = dataposition[24]
		self.temperature_trend_value = dataposition[25]
		self.max_temperature = dataposition[26]
		self.time_max_temperature = dataposition[27]
		self.minimum_temperature = dataposition[28]
		self.time_minimum_temperature = dataposition[29]
		self.max_wind_speed_today = dataposition[30]
		self.max_wind_speed_time = dataposition[31]
		self.max_wind_gust_today = dataposition[32]
		self.max_wind_gust_time = dataposition[33]
		self.max_pressure_today = dataposition[34]
		self.max_pressure_time = dataposition[35]
		self.min_pressure_today = dataposition[36]
		self.min_pressure_time = dataposition[37]
		self.cumulus_version = dataposition[38]
		self.cumulus_build_number = dataposition[39]
		self.ten_min_high_gust = dataposition[40]
		self.heat_index = dataposition[41]
		self.humidity_index = dataposition[42]

		if float(self.heat_index) > float(self.current_temperature ):
			self.feels_lable = "Heat Index"
			self.feels_data = self.heat_index
		else:
			self.feels_lable = "Wind Chill"
			self.feels_data = self.wind_chill

		if self.ang > self.target_angle:
			if (self.ang - self.target_angle) > 180:
				self.ang = self.ang - 360
		if self.ang < self.target_angle:
			if (self.target_angle - self.ang) > 180:
				self.ang = self.ang + 360

	#---------------------------
	# Read the semi static data
	#---------------------------
	def read_semistatic_data (self, line):
		"""
		Some data we need is not presented by realtime.txt, so we
		upload it via Cumulu's normal auto-update functionality
		This files splits on a pipe "|"
		Required fiels are: longitude,latitude,sunrise and sunset times
		and station name
		GEO position establish observation via Weatherunderground
		Celestial timing drives the sun or moon images
		<#longitude>|<#latitude>|<#sunrise>|<#sunset>|<#update>|<#location>
		"""
		dataposition = line.split('|')

		# Is it day or night?
		try:
			# Build strings to convert to time. Pre-append date to sunrise/set time
			# last_update = dataposition[4].split()
			sunrise_string = "01/01/01 " + dataposition[2]
			sunrise_time = time.strptime(sunrise_string, "%m/%d/%y %H:%M")
			print sunrise_time
			sunset_string = "01/01/01 " + dataposition[3]
			sunset_time = time.strptime(sunset_string, "%m/%d/%y %H:%M")
			print sunset_time

			latest_string = "01/01/01 " + self.lastupdate_time
			latest_time = time.strptime(latest_string, "%m/%d/%y %H:%M:%S")
			print latest_time


			# Is last update in daytime?
			if latest_time > sunrise_time and latest_time < sunset_time:
				self.back_image = self.srcdir + "/themes/" + self.theme_dir + "/symbols/Sun.png"
				print "It is daytime"
			else:
				self.get_moon()
				self.back_image = self.moon_image
				print "It is nightime"
		except:
			pass

		# Use GEO location to get conditions
		this_long = self.to_decimal_degrees(dataposition[0])
		this_lat = self.to_decimal_degrees(dataposition[1])
		self.url_wund = "http://www.wund.com/cgi-bin/findweather/getForecast?query=" + this_lat + "," + this_long
		# Set heading to Station name
		self.name = dataposition[5]
		print self.url_wund

	#---------------------------
	# Read the local observations from Weatherunderground
	#---------------------------
	def read_conditions_page (self, line):
		"""
		Weatherunderground has a nifty feature that alows you to
		query the observation closest to your GEO position by directly
		appending your longitude and lattitude to the wund url.
		Here we scrape the result for the <h4> tag which
		contains the observation.
		"""
		getCond = re.compile(r'<h4>(.*)</h4>', re.MULTILINE)
		self.conditions = getCond.search(line).groups()[0]
		print "WUND Says:" + self.conditions
		self.cim = self.Conditions[self.conditions]
		self.conditions_image = self.srcdir + "/themes/" + self.theme_dir + "/symbols/" + str(self.cim) + ".png"
		#self.conditions_image = self.srcdir + "/themes/" + self.theme_dir + "/symbols/" + "check.png"

	#---------------------------
	# Read the local observations from Weatherunderground
	#---------------------------
	def set_font (self):
		try:
			getFontsize = re.compile(r'(\d+)$')
			self.titlesize = int(getFontsize.search(self.titlefont).groups()[0])
			print "Font Size:", self.titlesize
		except:
			pass


	#---------------------------
	# Convert to decimal degrees
	#---------------------------
	def to_decimal_degrees (self, line):
		"""
		Converts the GEO position from DMS-format (as presented by Cumulus)
		to decimal degree format as required by Weatherunderground
		"""
		getDMS = re.compile('([N,S,E,W])\D+(\d+)\D+(\d+)\D+(\d+)')
		Q,D,M,S = getDMS.search(line).groups()
		if (Q == "S") or (Q == "W"):
			Q = (-1)
		else:
			Q = 1
		D = float(D)
		M = float(M)
		S = float(S)
		if D < 0 :
			DD = Q*(D + (M/-60) + (S/-3600))
		else:
			DD = Q*(D + (M/60) + (S/3600))
		return str(DD)

	#---------------------------
	# Moon stuff
	#---------------------------
	def get_moon(self):
		"""
		Almost verbatim from Marcus Furius's FuriusMoon Screenlet
		marcus_furius@hotmail.com
		"""
		today = datetime.date.today()
		year = int(today.strftime("%y"))
		month = int(today.strftime("%m"))
		day = int(today.strftime("%d"))

		description = ["new (totally dark)",
			"waxing crescent (increasing to full)",
			"first quarter (increasing to full)",
			"waxing gibbous (increasing to full)",
			"full (full light)",
			"waning gibbous (decreasing from full)",
			"last quarter (decreasing from full)",
			"waning crescent (decreasing from full)"]
		months = [" January ", " February ", " March ", " April ", " May ", " June ", " July ", " August ", " September ", " October ", " November ", " December "]

		ageOfMoon = year % 19
		if ageOfMoon > 9:
			ageOfMoon = (ageOfMoon - 19)

		ageOfMoon = (ageOfMoon * 11) % 30

		if month == 1:
			month = 3
		elif month == 2:
			month = 4
		else:
			month = month

		ageOfMoon = ageOfMoon + day + month - 8.3
		days_into_phase = ageOfMoon % 30

		index = int((days_into_phase + 2) * 16/59.0)
		if index > 7:
			index = 7
		status = description[index]

		# light should be 100% 15 days into phase
		light = int(2 * days_into_phase * 100/29)
		if light > 100:
			light = abs(light - 200)

		date = "%s%s%s" % (today.strftime("%d"), months[int(today.strftime("%m"))-1], today.strftime("%Y"))
		days_into_phase = int(days_into_phase)
		self.moon_image = self.srcdir + "/themes/" + self.theme_dir + "/moons/%s.png" %days_into_phase

	#---------------------------
	# Convert options selection (in string:minutes) to int:seconds
	#---------------------------
	def to_seconds (self, x):
		"""The Screenlet menu list only accept string values,
		so here we have to get the digit, convert it to an int
		and then convert to seconds. If the screenlet just started
		the value is undefined, so we try, and then except by setting
		the value to the first list item."""
		try:
			pattern = re.compile(r'(\d+)')
			time = pattern.search(x).groups()
			timer = int(time[0])*60
			return timer
		except:
			self.update_interval = self.update_choices[0]
			return 1*60

	#---------------------------
	# Draw rounded rectangle
	#---------------------------
	def draw_round_rect( self, ctx, x, y, w, h, r=(), fill=True ):
		ctx.move_to( x, y+r[0] )
		ctx.arc( x+r[0], y+r[0], r[0], 3.141, 4.712) #top left
		ctx.line_to( x-r[1]+w, y )
		ctx.arc( x-r[1]+w, y+r[1], r[1], 4.712, 0) #top right
		ctx.line_to( x+w, y-r[2]+h )
		ctx.arc( x-r[2]+w, y-r[2]+h, r[2], 0, 1.570) #bottom right
		ctx.line_to( x+r[3], y+h )
		ctx.arc( x+r[3], y-r[3]+h, r[3], 1.570, 3.141) #bottom left
		ctx.line_to( x, y+r[0] )
		if fill: ctx.fill()
		else: ctx.stroke()

	#---------------------------
	# Draw Text
	#---------------------------
	def draw_text( self, ctx, x, y, text, nowFont, size, rgba, align=0 ):
		"""
		align 0 = left
		align 1 = right
		align 2 = center
		"""
		ctx.save()
		if self.p_layout == None: self.p_layout = ctx.create_layout()
		else: ctx.update_layout(self.p_layout)
		p_fdesc = pango.FontDescription(nowFont)
		#p_fdesc.set_family_static()
		p_fdesc.set_size(size*pango.SCALE)
		self.p_layout.set_font_description(p_fdesc)
		self.p_layout.set_markup(str(text))
		ctx.set_source_rgba(rgba[0], rgba[1], rgba[2], rgba[3])
		textSize = self.p_layout.get_pixel_size()
		self.tsw = textSize[0]
		if align:
			if align == 1: x = x - textSize[0]
			elif align == 2: x = x - textSize[0]/2
		ctx.translate(x, y)
		ctx.show_layout(self.p_layout)
		ctx.fill()
		ctx.restore()

## If the program is run directly or passed as an argument to the python
## interpreter then create a Screenlet instance and show it
if __name__ == "__main__":
	import screenlets.session
	screenlets.session.create_session(SunbirdScreenlet)